@@ -27,6 +27,7 @@ Copyright 2014-2021 Sylvain Meignier and Anthony Larcher
     :mod:`features_server` provides methods to manage features
 
 """
+import copy
 import multiprocessing
 import numpy
 import logging
@@ -37,6 +38,7 @@ from sidekit.frontend.io import read_hdf5_segment
 from sidekit.frontend.vad import label_fusion
 from sidekit.frontend.normfeat import cms, cmvn, stg, cep_sliding_norm, rasta_filt
 from sidekit.sv_utils import parse_mask
+from sidekit.features_extractor import FeaturesExtractor
 
 
 __license__ = "LGPL"
@@ -219,7 +221,7 @@ class FeaturesServer(object):
             feat = pca_dct(feat, self.dct_pca_config[0], self.dct_pca_config[1], self.dct_pca_config[2])
         elif self.sdc:
             feat = shifted_delta_cepstral(feat, d=self.sdc_config[0], p=self.sdc_config[1], k=self.sdc_config[2])
-        
+    
         # Apply a mask on the features
         if self.mask is not None:
             feat = self._mask(feat)
@@ -480,13 +482,85 @@ class FeaturesServer(object):
                                                                  label=label,
                                                                  start=start, stop=stop,
                                                                  global_cmvn=self.global_cmvn)
+
         # Post-process the features and return the features and vad label
         if global_cmvn:
             feat, label = self.post_processing(feat, label, global_mean, global_std)
         else:
             feat, label = self.post_processing(feat, label)
+
         return feat, label
 
+    def get_features_per_speaker(self, show, idmap, channel=0, input_feature_filename=None, label=None):
+        """
+        Load a single file and return a dictionary with spk_ids as keys and (feature, label) as data
+        :param show:
+        :param channel:
+        :param input_feature_filename:
+        :param label:
+        :param idmap:
+        :return:
+        """
+        if input_feature_filename is not None:
+            self.feature_filename_structure = input_feature_filename
+
+        # If no extractor for this source, open hdf5 file and return handler
+        if self.features_extractor is None:
+            h5f = h5py.File(self.feature_filename_structure.format(show), "r")
+
+        # If an extractor is provided for this source, extract features and return an hdf5 handler
+        else:
+            h5f = self.features_extractor.extract(show, channel, input_audio_filename=input_feature_filename)
+
+        tmp_dict = dict()
+        for spk_id, start, stop in zip(idmap.leftids, idmap.start, idmap.stop):
+            if spk_id not in tmp_dict:
+                tmp_dict[spk_id] = numpy.arange(start, stop - 1)
+            else:
+                tmp_dict[spk_id] = numpy.concatenate((tmp_dict[spk_id], numpy.arange(start, stop - 1)), axis=0)
+
+        feat, lbl, global_mean, global_std, global_cmvn = read_hdf5_segment(h5f,
+                                                                            show,
+                                                                            dataset_list=self.dataset_list,
+                                                                            label=label,
+                                                                            start=None, stop=None,
+                                                                            global_cmvn=self.global_cmvn)
+
+        fe = FeaturesExtractor(audio_filename_structure="",
+                               feature_filename_structure=None,
+                               sampling_frequency=16000,
+                               lower_frequency=133.3333,
+                               higher_frequency=6855.4976,
+                               filter_bank="log",
+                               filter_bank_size=40,
+                               window_size=0.025,
+                               shift=0.01,
+                               ceps_number=13,
+                               pre_emphasis=0.97,
+                               keep_all_features=True,
+                               vad='percentil',
+                               save_param=["energy", "cep", "vad"]
+                               )
+
+        feat_per_spk = dict()
+        for spk_id in tmp_dict.keys():
+            lbl1 = copy.deepcopy(lbl)
+            feat1 = copy.deepcopy(feat)
+            
+            _, threshold_id = fe._vad(None, feat1[tmp_dict[spk_id], 0], None, None)
+            label_id = feat1[:, 0] > threshold_id
+            lbl1[tmp_dict[spk_id]] = label_id[tmp_dict[spk_id]].flatten()
+
+            # Post-process the features and return the features and vad label
+            if global_cmvn:
+                tmp_feat, tmp_lbl = self.post_processing(feat1[tmp_dict[spk_id], :], lbl1[tmp_dict[spk_id]], global_mean, global_std)
+            else:
+                tmp_feat, tmp_lbl = self.post_processing(feat1[tmp_dict[spk_id], :], lbl1[tmp_dict[spk_id]])
+           
+            feat_per_spk[spk_id] = (tmp_feat, tmp_lbl)
+
+        return feat_per_spk
+
     def get_tandem_features(self, show, channel=0, label=None, start=None, stop=None):
         """
         Read acoustic parameters from multiple HDF5 files (from disk or extracted by FeaturesExtractor objects).
@@ -523,7 +597,7 @@ class FeaturesServer(object):
 
         :param show: the ID of the show
         :param channel: the index of the channel
-        :param start: index of the first frame of the selected segment
+        param start: index of the first frame of the selected segment
         :param stop: index of the last frame of the selected segment
 
         :return: the number of frames, the mean of the frames and their standard deviation
