@@ -729,6 +729,54 @@ class StatServer:
                 # Compute 1st-order statistics
                 self.stat1[idx, :] = numpy.reshape(numpy.transpose(numpy.dot(data.transpose(), pp)), ubm.sv_size()).astype(STAT_TYPE)
 
+    def accumulate_stat_per_speaker(self, ubm, input_feature_filename, idmap, feature_server, channel_extension=("", "_b")):
+        """
+
+        :param ubm:
+        :param show:
+        :param input_feature_filename:
+        :param idmap:
+        :param feature_server:
+        :param channel_extension:
+        :return:
+        """
+        assert isinstance(ubm, Mixture), 'First parameter has to be a Mixture'
+        assert isinstance(feature_server, FeaturesServer), 'Second parameter has to be a FeaturesServer'
+
+        #show = self.segset[0]
+        show = input_feature_filename
+
+        # If using a FeaturesExtractor, get the channel number by checking the extension of the show
+        channel = 0
+        if feature_server.features_extractor is not None and show.endswith(channel_extension[1]):
+            channel = 1
+
+        show = show[:show.rfind(channel_extension[channel])]
+
+        features_per_speaker = feature_server.get_features_per_speaker(show, idmap, channel)
+
+        print(f"S0 a {features_per_speaker['S0'][1].sum()} trames")
+
+
+        for idx, (spk_id, data) in enumerate(features_per_speaker.items()):
+            cep, vad = data
+            cep = cep[vad]
+
+            # Verify that frame dimension is equal to gmm dimension
+            if not ubm.dim() == cep.shape[1]:
+                raise Exception('dimension of ubm and features differ: {:d} / {:d}'.format(ubm.dim(), cep.shape[1]))
+            else:
+                if ubm.invcov.ndim == 2:
+                    lp = ubm.compute_log_posterior_probabilities(cep)
+                else:
+                    lp = ubm.compute_log_posterior_probabilities_full(cep)
+                pp, foo = sum_log_probabilities(lp)
+                # Compute 0th-order statistics
+                self.stat0[idx, :] = pp.sum(0)
+                # Compute 1st-order statistics
+                self.stat1[idx, :] = numpy.reshape(numpy.transpose(numpy.dot(cep.transpose(), pp)),
+                                                   ubm.sv_size()).astype(STAT_TYPE)
+
     def get_mean_stat1(self):
         """Return the mean of first order statistics
         
@@ -1991,5 +2039,3 @@ class StatServer:
             statserver.stat1 = h5f[prefix+"stat1"][()][idx, :]
 
             return statserver
-
-
